#!/usr/bin/perl

# for x-fold training.
$folds = 1; # FOR EXAMPLE DATA. (I did 10-Fold in my experiments.)

#GENERATE TEST DATA
print STDERR "generating test data\n";
for( $i=0; $i<$folds; $i++ ) {
    print STDERR "\rPart $i\r";
    open(file, "data/aligned_Naive_german_celex.dat.part$i");
    @test = <file> ;
    open(TEST, ">data/test.voc$i");
    open(TESTRel, ">data/test.rel$i");
    foreach (@test){
	/^(\S+)\t(\S+)\s*/; 
	$w = $1; 
	$w =~ s/\#/1/g;
	$w=~s/(.)/$1 /g; 
	$w =~s/\_/\?/g; 
	$w=~s/\ \ +/\ /g; 
	print TEST $w."\n";
    }
    close(TEST);
    
    foreach(@test){
	/^(\S+)\t(\S+)\s*/; 
	$w = $2; 
	$w =~ s/\#/1/g; 
	$w=~s/(.)/$1 /g; 
	$w =~s/\_//g; 
	$w=~s/\-//g; 
	$w=~s/\  +/\ /g;
	$w=~s/^ +//;
	print TESTRel $w."\n";
    }
    close(TESTRel);
}

# PUT TRAINING DATA INTO RIGHT FORMAT AND TRAIN
print STDERR "generating training data and training\n";
my  @alltrain;
for ($i=0; $i<$folds;$i++){
    print STDERR "\rPart $i\r";
    # CREATE CORRECT FORMAT
    my @train = ();
    open(file, "data/aligned_Naive_german_celex.dat.part$i");
    @train= <file>;
    open(TRAIN, ">data/train$i");
    foreach (@train){
	/^(\S+)\t(\S+)\s*/; 
	$g = $1; 
	$p = $2;  
	$g =~ s/(.)/$1 /g;  
	@gr = split / /, $g;  
	$p =~ s/(.)/$1 /g;  
	@ph = split / /, $p;  
	for (my $i=0; $i<= $#gr; $i++){
	    $ph[$i] =~ s/\#/1/g;  
	    $ph[$i]=~ s/\_//;  
	    $ph[$i]=~s/\-//;  
	    $gr[$i]=~ s/\_/\?/g;  
	    print TRAIN "$gr[$i]-$ph[$i] ";  
	    push @alltrain, "$gr[$i]-$ph[$i] ";  
	}  
	print TRAIN "\n";  
    }
    close(TRAIN);
    
    # for each fold, create a file that contains all the training data 
    # except the test data for that fold
    for( $k=0; $k < $folds; $k++ ) {
	if ($i != $k){
	    system("cat data/train$k > data/train$i.$folds"."fold");
	}
    }
    
    # NOTE: this means you'll be testing on your training data!!!
    if ($folds == 1){
	print STDERR "TRAINING DATA = TEST DATA!\n";
	system("mv data/train$i data/train$i.1fold");
    }

    # TRAIN
    system("perl hmm-g2p-train.pl -p 4 data/train$i.$folds"."fold > data/par4ger.$i");
}

# GENERATE THE LIST OF POSSIBLE MAPPINGS FROM GRAPHEMES TO PHONEMES:

open(MAPPING, ">data/phon_graph_ger.txt");
print STDERR "creating mapping file \n";
foreach (@alltrain){
    @a = split;  
    foreach $b (@a) {
	$map{$b}++;  
    }
}

foreach $l (sort keys %map) {
    $l=~/^(.)\-(.*)$/;  
    print MAPPING "letter-output: "."\L$1"." --> $2\n";  
						 
}

# RUN THE LETTER-TO-PHONEME CONVERSION ALGORITHM:
print STDERR "running conversion\n";
for ($i=0; $i<$folds;$i++){
    print STDERR "PART $i\n";
    system("cat data/test.voc$i | nice perl hmm-g2p-tag-backoff2.pl data/phon_graph_ger.txt data/par4ger.$i > data/test$i.res");
}

print STDERR "DONE\n";

# EVALUATION
print STDERR "EVALUATION\n";
# WORD ERROR RATE:
system("echo \"\" > eval.txt");
system("echo \"\#WORD-ERROR-RATE_\" > eval.txt");
for($i = 0; $i < $folds; $i++){
    #system('echo "\#fold $i\n" > eval.txt');
    system("diff -b data/test$i.res data/test.rel$i | grep '<' | wc -l >> eval.txt");
    system("echo '/' >> eval.txt");
    system("wc -l data/test.rel$i >> eval.txt");
    system("echo '_' >> eval.txt");
}

# PHONEME ERROR RATE:
system("echo \"\n\#PHONEME-ERROR-RATE_\" >> eval.txt");
for($i = 0; $i < $folds; $i++)
{
    #system('echo "\#fold $i\n" > eval.txt');
    system("perl -pe 's/ /\n/g' data/test.rel$i > tl$i");
    system("perl -pe 's/ /\n/g' data/test$i.res > ts$i");
    system("echo '(' >> eval.txt");
    system("diff -b ts$i tl$i | grep -c '<' >> eval.txt");
    system("echo '+' >> eval.txt");
    system("diff -b ts$i tl$i | grep -c '>' >> eval.txt");
    system("echo ')/' >> eval.txt");
    system("wc -l tl$i >> eval.txt");
    system("echo '_' >> eval.txt");
   # system("rm tl$i ts$i");
#system("diff -b test$i.res test.rel$i | grep '>' | wc -l");
}

system("perl -pe 's/ .*//; s/\n/ /; s/_/\n/; ' eval.txt |  perl -pe 's/^ +//;' > evaluation_result.txt");
system("rm eval.txt");
open(ERG, "evaluation_result.txt");
print STDERR "\n";
while(<ERG>){
    if ($_ =~ s/^\#//){
	print STDERR $_;
    }
    else {
	system("perl -e 'printf \"%2.4f\n\", $_;'");
    }
}
close(ERG);
